#include "Arduino.h"
#include "CaptivePortalSpiffsServer.h"

// Allocate memory for static variables
String CaptivePortalSpiffsServer::_host;
DNSServer CaptivePortalSpiffsServer::_dnsServer;
WebServer *CaptivePortalSpiffsServer::_webServer;

// Returns the webServer pointer
void CaptivePortalSpiffsServer::init(WebServer *webServer) {
	if (_webServer == NULL) _webServer = webServer;			// Initialise webServer if not yet done
}

// Begin => to set up everything
void CaptivePortalSpiffsServer::begin(String host, char *ssid, char *passwd, String rootFolder) {
	_host = host;
	if (_webServer == NULL) _webServer = new WebServer(80);												// Initialise webServer if not yet done
	
	if (WiFi.getMode() != WIFI_AP && WiFi.getMode() != WIFI_AP_STA) WiFi.mode(WIFI_AP);
	WiFi.softAPConfig(IPAddress(1, 1, 1, 1), IPAddress(1, 1, 1, 1), IPAddress(255, 255, 255, 0));		// Configure ip
	WiFi.softAP(ssid, passwd);																			// Set WiFi name & password
	
	_dnsServer.start(53, "*", IPAddress(1, 1, 1, 1));
	
	SPIFFS.begin(true);														// Start SPIFFS file system
	File root = SPIFFS.open("/");											// Open the root folder
	
	while (File file = root.openNextFile()) {								// Go through each file
		if (!file.isDirectory()) {											// Skip folders
			String fileName = file.name();
			
			if (fileName.startsWith(rootFolder)) {
				String fileType = getFileType(fileName);
				String onFileName = fileName.substring(rootFolder.length()-1, fileName.indexOf("index", fileName.lastIndexOf("/")));
				
				_webServer->on(onFileName, HTTP_GET, [fileName, fileType] {		// Create a webServer link for file
					if (_webServer->header("Host") != _host) {
						toIndex();												// Redirect to index if host doesn't match
					} else {
						_webServer->sendHeader("Cache-Control", "max-age=0");	// Do not cache files
						
						File file = SPIFFS.open(fileName, FILE_READ);
							_webServer->streamFile(file, fileType);				// Stream File to browser
						file.close();
					}
				});
			}
		}
	}
	
	_webServer->onNotFound(toIndex);											// If not found send to index (opens captive portal)
	
	const char* headerKeys[] = {"Host"};										// The SERVER keys to collect
	const size_t headerKeysSize = sizeof(headerKeys) / sizeof(char*);
	_webServer->collectHeaders(headerKeys, headerKeysSize);						// To check if host matches the given name
	
	_webServer->begin();														// Start webServer
}

// Loop => to process dns & server
void CaptivePortalSpiffsServer::loop() {
	_dnsServer.processNextRequest();					// Handle dnsServer
	_webServer->handleClient();							// Handle webServer
}

// Get file type
String CaptivePortalSpiffsServer::getFileType(String fileName) {
	String ending = fileName.substring(fileName.lastIndexOf(".") + 1);
	
	if (ending == "html")	return "text/html";
	if (ending == "css")	return "text/css";
	if (ending == "js")		return "text/javascript";
	if (ending == "json")	return "application/json";
	if (ending == "svg")	return "image/svg+xml";
	
	return "text/plain";
}

// HTTP redirect to index
void CaptivePortalSpiffsServer::toIndex() {
	_webServer->sendHeader("Location", "http://" + _host);
	_webServer->send(302);
}